/* -*- mode:c -*- */
/** @file
 ** @brief check corner cases to improve coverage reports
 **/

/*
 * Copyright (C) 2020 HPE, HP Schweiz GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* needed before inclusion of cheat.h: */
#ifndef __BASE_FILE__
#define __BASE_FILE__ __FILE__
#endif

#include "cheat.h"

#include "maestro.h"
#include "maestro/i_cdo.h"
#include <stdint.h>

#include "maestro/i_base64.h"
CHEAT_TEST(base64,
           {
             size_t dummy;
             const unsigned char *str = (const unsigned char*)"foo";
             cheat_assert(NULL==base64_encode(str, SIZE_MAX,&dummy));
             /* hopefully no one has that much memory... */
             cheat_assert(NULL==base64_encode(str, SIZE_MAX/3 ,&dummy));
           })
    
    
    CHEAT_TEST(base64decode,
               {
                 size_t dummy;
                 cheat_assert(NULL==base64_decode(NULL, 0,&dummy));
                 cheat_assert(NULL==base64_decode((const unsigned char*)"===", 3, &dummy));
               })

           
CHEAT_TEST(null_cdo, { 
		cheat_assert(MSTRO_CDO_STATE_INVALID==mstro_cdo_state_get(NULL)) ;
		cheat_assert(NULL==mstro_cdo_name(NULL));
		})

CHEAT_TEST(invalid_cdo_declare, {
	mstro_cdo res;
	cheat_assert(MSTRO_OK == mstro_init("Tests","Init",0));
	cheat_assert(MSTRO_INVARG==mstro_cdo_declare(NULL, MSTRO_ATTR_DEFAULT, &res));
	cheat_assert(MSTRO_INVOUT==mstro_cdo_declare("foo", MSTRO_ATTR_DEFAULT, NULL));

	cheat_assert(NULL==MSTRO_ATTR_DEFAULT);
	mstro_cdo_attributes really_invalid_attr=(mstro_cdo_attributes)(1);
	cheat_assert(MSTRO_UNIMPL==mstro_cdo_declare("foo", really_invalid_attr, &res));
	cheat_assert(MSTRO_OK == mstro_finalize());
	})

CHEAT_TEST(invalid_cdo_seal, {
	cheat_assert(MSTRO_INVARG==mstro_cdo_declaration_seal(NULL));
			})

CHEAT_TEST(invalid_cdo_offer, {
	cheat_assert(MSTRO_INVARG==mstro_cdo_offer(NULL));
})

CHEAT_TEST(invalid_cdo_withdraw, {
	cheat_assert(MSTRO_INVARG==mstro_cdo_withdraw(NULL));
})

CHEAT_TEST(invalid_cdo_dispose, {
	cheat_assert(MSTRO_INVARG==mstro_cdo_dispose(NULL));
})
